
/*
 * $Revision: 1.2 $
 */
 
#include "config.h"

#define ROM_OFFSET 0x80  // Start of ROM monitor exception vector table
#define IRQ_PTR      (0x0038 + ROM_OFFSET) // Storage location for address of IRQ handler.
#define FIQ_PTR      (0x003C + ROM_OFFSET) // Storage location for address of FIQ handler.
#define IRQ_VECTOR   (0x0018 + ROM_OFFSET) // IRQ Vector location.
#define FIQ_VECTOR   (0x001C + ROM_OFFSET) // FIQ Vector location.

static int *button_value_ptr;
static void (*timer_function)();
static int timer_block;


//
// Interrupt handlers.
//

// IRQ interrupt handler.
// Handles the timer interrupt and the external interrupt 0 (PUSH_SW3).
__irq __arm void irq_handler(void)
{
  // Called at 1000 Hz rate.
  if(__IRLR != 0x00) // Interrupt level.
  {
    switch(__INR) // Interrupt number.
    {
    case 16:// External interrupt 0, button PUSH_SW3.
      // Toggle the button value.
      *button_value_ptr = !*button_value_ptr;
      // Clear interrupt.
      __CILR = 0xFE; // Reset all interrupt levels.
      __IRR1 = 0x01; // Clear external interrupt 0.
      break;

    case 8: // Timer TMOV0 interrupt
      if (timer_block)
      {
        timer_block--; // Count down while blocking timer callback.
      }
      else
      {
        (*timer_function)(); // Call timer callback function.
      }
      __TM0ST = 0x03; // Clear timer EVENT and OVF bits.
      // Clear interrupt.
      __CILR = 0xFE; // Reset all interrupt levels.
      __IRR0 = 0x0100; // Clear timer 0 interrupt
      break;

    default:
      // Recover from unknown interrupt.
      __IRR0 = 0x0100; // Clear timer 0 interrupt
      __CILR = 0xFE;  // Reset all interrupt levels
      break;
    }
  }
}


// FIQ interrupt handler.
// Handles FIQ button (PUSH_SW2).
__fiq __arm void fiq_handler(void)
{
  __EFIQCON = 1; // Clear EFIQR.
  JOBLedSet(0xff); // All LED's on.
  timer_block = 1000; // Block LED flashing for 1 second.
}


// Help function to install interrupt handlers.
// The contents of 'vector' are made to contain an 'LDR pc,[pc, #offset]'
// instruction which, when operated by an interrupt being sensed, will cause
// a long branch to the address in 'location'. The return value is the
// contents of 'vector', ie. the previous vector.
static unsigned InstallHandler (unsigned *location, unsigned *vector)
{
  unsigned vec, oldvec;
  
  // Create new vector value using LDR pc ,[pc,#offs] where offs is
  // calculated from the address at 'location'.
  vec = ((unsigned)location - (unsigned)vector - 0x08) | 0xe59ff000;

  oldvec = *vector; // Save original contents of vector location.
  *vector = vec; // Store new vector information at location.

  return oldvec; // Return old vector information.
}


//
// Interrupt functions.
//

void JOBInitInterrupt(void(*timer_func)())
{
  unsigned *irqvec  = (unsigned *)IRQ_VECTOR;
  unsigned *fiqvec  = (unsigned *)FIQ_VECTOR;
  unsigned *irqaddr = (unsigned *)IRQ_PTR;
  unsigned *fiqaddr = (unsigned *)FIQ_PTR;

  // Setup timer callback function.
  timer_function = timer_func;

  // Set up IRQ interrupt vector.
  *irqaddr = (unsigned)irq_handler; // Create a pointer to IRQ Handler
  InstallHandler(irqaddr, irqvec); // and install the handler.
  
  // Set up FIQ interrupt vector.
  *fiqaddr = (unsigned)fiq_handler; // Create a pointer to FIQ Handler
  InstallHandler(fiqaddr, fiqvec); // and install the handler.
}


//
// Timer functions.
//

void JOBInitTimer()
{
  __TM0CON = 0x04; // Auto reload mode, MCLK/16 prescaler.
  __TM0C = (0xFFFF - JOB_MCLK/16/1000); // 1 ms period time.
  
  __ILCON2 |= 0x0003; // Interrupt level for timer TMOV0.
}

void JOBStartTimer()
{
  __TMEN |= 0x01; // Start timer TMOV0.
}


//
// Parallel I/O functions.
//

void JOBInitPIO(int *button_ptr)
{
  button_value_ptr = button_ptr;

  __ILCON4 |= 0x0003; // Interrupt level for external interrupt 0.
  
  __PO3 =0x0000; // Clear all 8 LED's.
  __PM3 =0xff00; // Set P7 to have all bits as output (LED's).
}


//
// LED output drivers.
//

void JOBLedSet(unsigned int mask)
{
  __PO3 &= 0x00ff; // Clear all 8 LED's.
  __PO3 |= mask << 8; // Set LED's.
}


